// 001 u_c_xxx
// 04 mar 00

(*$r+*)

unit u_c_object_inspector;
  interface

    // uses u_c_basic_object;
    uses FMX.Controls
        , System.UITypes // mouse buttons
        , System.Classes // TShiftState

        , FMX.Edit

        ;

    const k_property_value_x= 80+ 60+ 2;
          k_value_width= 100;

    type c_fmx_object_inspector=
             class
               m_c_property_panel_ref: tPanel;
               m_c_property_in_place_edit_ref: tEdit;
               m_property_count: Integer;
               m_c_created_component_ref: tComponent;
               m_y: Double;

               m_on_change_notify: tNotifyEvent;

               Constructor create_fmx_object_inspector(
                   p_c_property_panel_ref: tPanel;
                   p_c_property_in_place_edit_ref: tEdit;
                   p_on_change_notify: tNotifyEvent);

               procedure add_fmx_property(p_property_name, p_property_value: String);
               procedure detach_in_place_edit;
               procedure display_in_place_edit;
               procedure clear_object_inspector;
               procedure fill_fmx_object_properties(p_c_component: tComponent);

               procedure handle_property_value_mousedown(Sender: TObject; Button: TMouseButton;
                   Shift: TShiftState; X, Y: Single);
               procedure handle_property_in_place_edit_KeyDown(Sender: TObject; var Key: Word;
                   var KeyChar: Char; Shift: TShiftState);

               Destructor Destroy; Override;
             end; // c_fmx_object_inspector

  implementation
    uses TypInfo, SysUtils
        , Fmx.Types, Fmx.Objects
        , u_display_xe2
        , u_display_fmx_object
        ;

    procedure display(p_text: String);
      begin
        // u_display_xe2.display(p_text);
      end; // display

    // -- c_fmx_object_inspector

    Constructor c_fmx_object_inspector.create_fmx_object_inspector(p_c_property_panel_ref: tPanel;
        p_c_property_in_place_edit_ref: tEdit;
        p_on_change_notify: tNotifyEvent);
      begin
        // Inherited create_basic_object(p_name);
        Inherited Create;
        m_c_property_panel_ref:= p_c_property_panel_ref;
        m_c_property_in_place_edit_ref:= p_c_property_in_place_edit_ref;
        m_c_property_in_place_edit_ref.OnKeyDown:= handle_property_in_place_edit_KeyDown;
        m_on_change_notify:= p_on_change_notify;
      end; // create_fmx_object_inspector

    procedure c_fmx_object_inspector.add_fmx_property(p_property_name, p_property_value: String);
      var l_c_property_name_label: tLabel;
          l_c_value_rectangle: tRectangle;
          l_c_property_value_text: tText;
          l_height: Single;
      begin
          l_height:= m_c_property_in_place_edit_ref.Height+ 2;

          l_c_property_name_label:= tLabel.Create(m_c_property_panel_ref);
          with l_c_property_name_label do
          begin
            Parent:= m_c_property_panel_ref;
            Position.X:= 5;
            Position.Y:= m_y+ 2;
            Text:= p_property_name;
            // -- debug
            Name:= Text+ '_name_label_'+ IntToStr(m_property_count);
          end; // with l_c_property_name_label

          l_c_value_rectangle:= tRectangle.Create(m_c_property_panel_ref);
          with l_c_value_rectangle do
          begin
            Parent:= m_c_property_panel_ref;
            Position.X:= k_property_value_x;
            Position.Y:= m_y;
            Width:= k_value_width;
            Stroke.Color:= claRed;
            Height:= l_height;

            // -- debug
            Name:= l_c_property_name_label.Text+ '_value_rectangle_'+ IntToStr(m_property_count);
          end; // with l_c_value_rectangle

          l_c_property_value_text:= tText.Create(m_c_property_panel_ref);
          with l_c_property_value_text do
          begin
            Parent:= l_c_value_rectangle;
            Position.X:= 4;
            Position.Y:= 1;
            Width:= k_value_width;
            Height:= l_height- 2;
            HorzTextAlign:= TTextAlign.taLeading;
            Text:= p_property_value;

            // -- attach the label to the rTextcomponent
            // --   to be able to find the property name from the tText
            Tag:= NativeInt(l_c_property_name_label);

            // -- the font color
            Fill.Color:= claDarkTurquoise;

            OnMouseDown:= handle_property_value_mousedown;
          end; // with l_c_property_value_text

        m_y:= m_y+ l_height+ 2;
        Inc(m_property_count);
      end; // add_fmx_property

    procedure c_fmx_object_inspector.detach_in_place_edit;
      begin
        with m_c_property_in_place_edit_ref do
        begin
          // -- The tEdit becomes invisible
          Parent:= m_c_property_panel_ref;
          Tag:= 0;
          Visible:= False;
        end; // with Form1.m_c_property_in_place_edit_ref
      end; // detach_in_place_edit

    procedure c_fmx_object_inspector.display_in_place_edit;
      var l_integer: NativeInt;
      begin
        if m_c_property_in_place_edit_ref= Nil
          then display('NIL')
          else display(Format('5.2f', [m_c_property_in_place_edit_ref.Width]));

        l_integer:= NativeInt(m_c_property_in_place_edit_ref.Parent);
        display('in_place_edit_.Parent $'+ IntToHex(l_integer, 4));
      end; // display_in_place_edit

    procedure c_fmx_object_inspector.clear_object_inspector;
      var l_child_index: Integer;
          l_c_fmx_object: tFmxObject;
      begin
        // -- do not remove if not yet filled
        if m_c_property_panel_ref.ChildrenCount= 1
          then Exit;

        display('> === clear');

        display_in_place_edit;

        // -- if had clicked a value, the in_place_edit had becomea CHILD of the tRectangle
        detach_in_place_edit;

        with m_c_property_panel_ref do
        begin
          display('child_0_classname '+ Children[0].ClassName);
          display(f_display_fmx_object_children_max(m_c_property_panel_ref, 1));
          l_child_index:= ChildrenCount- 1;
          while l_child_index>= 0 do
          begin
            l_c_fmx_object:= Children[l_child_index];
            display(Format('%3d ', [l_child_index])+ l_c_fmx_object.Name);
            if ((l_c_fmx_object is tLabel)
                or (l_c_fmx_object is tRectangle)) and (l_c_fmx_object.Name<> '')
              then begin
                  display('  erase '+ l_c_fmx_object.Name);
                  RemoveObject(l_c_fmx_object);
                  FreeAndNil(l_c_fmx_object);
                end;

            Dec(l_child_index);
          end; // while l_child_index
        end; // with Form1.m_c_property_panel_ref

        display_in_place_edit;
        display('< === clear');
      end; // clear_object_inspector

    procedure c_fmx_object_inspector.fill_fmx_object_properties(p_c_component: tComponent);

      procedure get_properties_using_typinfo;
        var l_pt_property_list: PPropList;
            l_property_count : Integer;
            l_property_index : Integer;
            l_property_name, l_property_value: String;
        begin
          GetMem(l_pt_property_list, SizeOf(Pointer) * GetTypeData(p_c_component.ClassInfo)^.PropCount);

          try
            l_property_count := GetPropList(p_c_component.ClassInfo,tkProperties, l_pt_property_list,true);

            for l_property_index:= 0 to l_property_count- 1 do
            begin
              l_property_name:= l_pt_property_list^[l_property_index].Name;
              l_property_value:= GetPropValue(p_c_component, l_property_name, True);

              add_fmx_property(l_property_name, l_property_value);
            end; // for l_property_index
          finally
            FreeMem(l_pt_property_list);
          end;
        end; // get_properties_using_typinfo

      begin // fill_fmx_object_properties
        clear_object_inspector;
        m_y:= 2;
        m_c_created_component_ref:= p_c_component;
        get_properties_using_typinfo;

        m_c_property_panel_ref.Height:= m_y;
        // m_c_property_panel_ref.Repaint;
      end; // fill_fmx_object_properties

    procedure c_fmx_object_inspector.handle_property_value_mousedown(Sender: TObject; Button: TMouseButton;
        Shift: TShiftState; X, Y: Single);
      var l_c_sender_text: tText;
          l_c_property_value_rectangle: tRectangle;
      begin
        // display('down '+ (Sender as tLabel).Text);
        if not (Sender is tText)
          then display('*** not tText');

        l_c_sender_text:= Sender as tText;
        with l_c_sender_text, Position do
          display(Format('down %5.2f %5.2f', [X, Y])+ Text);

        l_c_property_value_rectangle:= l_c_sender_text.Parent as tRectangle;

        with l_c_property_value_rectangle, Position do
          display(Format('rect_parent %5.2f %5.2f ', [X, Y])+ Name);

        with m_c_property_in_place_edit_ref do
        begin
          Parent:= l_c_property_value_rectangle;
          // -- position relative to the PARENT
          Position.X:= 4;
          Position.Y:= 1;

          // -- attach the tText to the in place edit Tag
          Tag:= NativeInt(l_c_sender_text);

          Text:= 'aha';
          Text:= l_c_sender_text.Text;
          Visible:= True;
          display('edit_parent '+ Parent.Name);
        end; // with m_c_property_in_place_edit_ref
      end; // handle_property_value_mousedown

    procedure c_fmx_object_inspector.handle_property_in_place_edit_KeyDown(Sender: TObject; var Key: Word;
        var KeyChar: Char; Shift: TShiftState);
      var l_c_property_text: tText;
          l_selected_component_name: String;
          l_c_component: TComponent;
          l_c_property_name_label: tLabel;
      begin
        if Key= 13
          then begin
              display('in_place_enter');
              m_c_property_in_place_edit_ref.Visible:= False;

              l_c_property_text:= tText(m_c_property_in_place_edit_ref.Tag);
              l_c_property_text.Text:= m_c_property_in_place_edit_ref.Text;

              l_c_property_name_label:= tLabel(l_c_property_text.Tag);

              display('update_prop '+ l_c_property_name_label.Text);
              display('  new_value '+ m_c_property_in_place_edit_ref.Text);

              setPropValue(m_c_created_component_ref, l_c_property_name_label.Text, m_c_property_in_place_edit_ref.Text);

              // -- The tEdit becomes invisible
              detach_in_place_edit;

              // -- notify the Form
              if Assigned(m_on_change_notify)
                then m_on_change_notify(Self);
            end;
      end; // handle_property_in_place_edit_KeyDown

    Destructor c_fmx_object_inspector.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_fmx_object_inspector
    end. // u_c_fmx_object_inspector


